from typing import Any
from rapidata.rapidata_client.filter.base_filter import Filter
from rapidata.api_client.models.age_user_filter_model import AgeUserFilterModel
from rapidata.api_client.models.age_group import AgeGroup


class AgeFilter(Filter):

    def __init__(self, age_groups: list[AgeGroup]):
        self.age_groups = age_groups

    def to_model(self):
        return AgeUserFilterModel(
            _t="AgeFilter",
            ageGroups=self.age_groups,
        )



import os
from typing import Any
from rapidata.api_client.models.add_validation_rapid_model import (
    AddValidationRapidModel,
)
from rapidata.api_client.models.add_validation_text_rapid_model import (
    AddValidationTextRapidModel,
)
from rapidata.api_client.models.add_validation_rapid_model_payload import (
    AddValidationRapidModelPayload,
)
from rapidata.api_client.models.add_validation_rapid_model_truth import (
    AddValidationRapidModelTruth,
)
from rapidata.api_client.models.attach_category_truth import AttachCategoryTruth
from rapidata.api_client.models.bounding_box_payload import BoundingBoxPayload
from rapidata.api_client.models.bounding_box_truth import BoundingBoxTruth
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.compare_payload import ComparePayload
from rapidata.api_client.models.compare_truth import CompareTruth
from rapidata.api_client.models.datapoint_metadata_model_metadata_inner import (
    DatapointMetadataModelMetadataInner,
)
from rapidata.api_client.models.empty_validation_truth import EmptyValidationTruth
from rapidata.api_client.models.free_text_payload import FreeTextPayload
from rapidata.api_client.models.line_payload import LinePayload
from rapidata.api_client.models.line_truth import LineTruth
from rapidata.api_client.models.locate_box_truth import LocateBoxTruth
from rapidata.api_client.models.locate_payload import LocatePayload
from rapidata.api_client.models.named_entity_payload import NamedEntityPayload
from rapidata.api_client.models.named_entity_truth import NamedEntityTruth
from rapidata.api_client.models.polygon_payload import PolygonPayload
from rapidata.api_client.models.polygon_truth import PolygonTruth
from rapidata.api_client.models.transcription_payload import TranscriptionPayload
from rapidata.api_client.models.transcription_truth import TranscriptionTruth
from rapidata.api_client.models.transcription_word import TranscriptionWord
from rapidata.rapidata_client.assets.media_asset import MediaAsset
from rapidata.rapidata_client.assets.multi_asset import MultiAsset
from rapidata.rapidata_client.assets.text_asset import TextAsset
from rapidata.rapidata_client.metadata.base_metadata import Metadata
from rapidata.service.openapi_service import OpenAPIService


class RapidataValidationSet:
    """A class for interacting with a Rapidata validation set.

    Get a `ValidationSet` either by using `rapi.get_validation_set(id)` to get an existing validation set or by using `rapi.new_validation_set(name)` to create a new validation set.
    """

    def __init__(self, validation_set_id, openapi_service: OpenAPIService, name: str):
        self.id = validation_set_id
        self.openapi_service = openapi_service
        self.name = name

    def add_general_validation_rapid(
        self,
        payload: (
            BoundingBoxPayload
            | ClassifyPayload
            | ComparePayload
            | FreeTextPayload
            | LinePayload
            | LocatePayload
            | NamedEntityPayload
            | PolygonPayload
            | TranscriptionPayload
        ),
        truths: (
            AttachCategoryTruth
            | BoundingBoxTruth
            | CompareTruth
            | EmptyValidationTruth
            | LineTruth
            | LocateBoxTruth
            | NamedEntityTruth
            | PolygonTruth
            | TranscriptionTruth
        ),
        metadata: list[Metadata],
        asset: MediaAsset | TextAsset | MultiAsset,
        randomCorrectProbability: float,
    ) -> None:
        """Add a validation rapid to the validation set.

        Args:
            payload: The payload for the rapid.
            truths: The truths for the rapid.
            metadata (list[Metadata]): The metadata for the rapid.
            asset: The asset(s) for the rapid.
            randomCorrectProbability (float): The random correct probability for the rapid.

        Returns:
            None

        Raises:
            ValueError: If an invalid asset type is provided.
        """

        model = AddValidationRapidModel(
            validationSetId=self.id,
            payload=AddValidationRapidModelPayload(payload),
            truth=AddValidationRapidModelTruth(truths),
            metadata=[
                DatapointMetadataModelMetadataInner(meta.to_model())
                for meta in metadata
            ],
            randomCorrectProbability=randomCorrectProbability,
        )
        if isinstance(asset, MediaAsset):
            self.openapi_service.validation_api.validation_add_validation_rapid_post(
                model=model, files=[asset.path]
            )

        elif isinstance(asset, TextAsset):
            model = AddValidationTextRapidModel(
                validationSetId=self.id,
                payload=AddValidationRapidModelPayload(payload),
                truth=AddValidationRapidModelTruth(truths),
                metadata=[
                    DatapointMetadataModelMetadataInner(meta.to_model())
                    for meta in metadata
                ],
                randomCorrectProbability=randomCorrectProbability,
                texts=[asset.text],
            )
            self.openapi_service.validation_api.validation_add_validation_text_rapid_post(
                add_validation_text_rapid_model=model
            )

        elif isinstance(asset, MultiAsset):
            files = [a.path for a in asset if isinstance(a, MediaAsset)]
            texts = [a.text for a in asset if isinstance(a, TextAsset)]
            if files:
                self.openapi_service.validation_api.validation_add_validation_rapid_post(
                    model=model, files=files # type: ignore
                )
            if texts:
                model = AddValidationTextRapidModel(
                    validationSetId=self.id,
                    payload=AddValidationRapidModelPayload(payload),
                    truth=AddValidationRapidModelTruth(truths),
                    metadata=[
                        DatapointMetadataModelMetadataInner(meta.to_model())
                        for meta in metadata
                    ],
                    randomCorrectProbability=randomCorrectProbability,
                    texts=texts,
                )
                self.openapi_service.validation_api.validation_add_validation_text_rapid_post(
                    add_validation_text_rapid_model=model
                )


        else:
            raise ValueError("Invalid asset type")

    def add_classify_rapid(
        self,
        asset: MediaAsset | TextAsset,
        question: str,
        categories: list[str],
        truths: list[str],
        metadata: list[Metadata] = [],
    ) -> None:
        """Add a classify rapid to the validation set.

        Args:
            asset (MediaAsset | TextAsset): The asset for the rapid.
            question (str): The question for the rapid.
            categories (list[str]): The list of categories for the rapid.
            truths (list[str]): The list of truths for the rapid.
            metadata (list[Metadata], optional): The metadata for the rapid. Defaults to an empty list.

        Returns:
            None
        """
        payload = ClassifyPayload(
            _t="ClassifyPayload", possibleCategories=categories, title=question
        )
        model_truth = AttachCategoryTruth(
            correctCategories=truths, _t="AttachCategoryTruth"
        )

        self.add_general_validation_rapid(
            payload=payload,
            truths=model_truth,
            metadata=metadata,
            asset=asset,
            randomCorrectProbability=len(truths) / len(categories),
        )

    def add_compare_rapid(
        self,
        asset: MultiAsset,
        question: str,
        truth: str,
        metadata: list[Metadata] = [],
    ) -> None:
        """Add a compare rapid to the validation set.

        Args:
            asset (MultiAsset): The assets for the rapid.
            question (str): The question for the rapid.
            truth (str): The path to the truth file.
            metadata (list[Metadata], optional): The metadata for the rapid. Defaults to an empty list.

        Returns:
            None

        Raises:
            ValueError: If the number of assets is not exactly two.
        """
        payload = ComparePayload(_t="ComparePayload", criteria=question)
        # take only last part of truth path
        truth = os.path.basename(truth)
        model_truth = CompareTruth(_t="CompareTruth", winnerId=truth)

        if len(asset) != 2:
            raise ValueError("Compare rapid requires exactly two media paths")

        self.add_general_validation_rapid(
            payload=payload,
            truths=model_truth,
            metadata=metadata,
            asset=asset,
            randomCorrectProbability=1 / len(asset),
        )

    def add_transcription_rapid(
        self,
        asset: MediaAsset | TextAsset,
        question: str,
        transcription: list[str],
        correct_words: list[str],
        strict_grading: bool | None = None,
        metadata: list[Metadata] = [],
    ) -> None:
        """Add a transcription rapid to the validation set.

        Args:
            asset (MediaAsset | TextAsset): The asset for the rapid.
            question (str): The question for the rapid.
            transcription (list[str]): The transcription for the rapid.
            correct_words (list[str]): The list of correct words for the rapid.
            strict_grading (bool | None, optional): The strict grading for the rapid. Defaults to None.
            metadata (list[Metadata], optional): The metadata for the rapid. Defaults to an empty list.

        Returns:
            None

        Raises:
            ValueError: If a correct word is not found in the transcription.
        """
        transcription_words = [
            TranscriptionWord(word=word, wordIndex=i)
            for i, word in enumerate(transcription)
        ]

        correct_transcription_words = []
        for word in correct_words:
            if word not in transcription:
                raise ValueError(f"Correct word '{word}' not found in transcription")
            correct_transcription_words.append(
                TranscriptionWord(word=word, wordIndex=transcription.index(word))
            )

        payload = TranscriptionPayload(
            _t="TranscriptionPayload", title=question, transcription=transcription_words
        )

        model_truth = TranscriptionTruth(
            _t="TranscriptionTruth",
            correctWords=correct_transcription_words,
            strictGrading=strict_grading,
        )

        self.add_general_validation_rapid(
            payload=payload,
            truths=model_truth,
            metadata=metadata,
            asset=asset,
            randomCorrectProbability=len(correct_words) / len(transcription),
        )



import json
import time
import requests
import threading
from rapidata.api_client.api.campaign_api import CampaignApi
from rapidata.api_client.api.dataset_api import DatasetApi
from rapidata.api_client.api.order_api import OrderApi
from rapidata.api_client.api.pipeline_api import PipelineApi
from rapidata.api_client.api.rapid_api import RapidApi
from rapidata.api_client.api.validation_api import ValidationApi
from rapidata.api_client.api.workflow_api import WorkflowApi
from rapidata.api_client.api_client import ApiClient
from rapidata.api_client.configuration import Configuration


class OpenAPIService:

    _TOKEN_EXPIRATION_MINUTES = 30

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        endpoint: str,
        token_url: str,
        oauth_scope: str,
        cert_path: str | None = None
    ):
        client_configuration = Configuration(host=endpoint, ssl_ca_cert=cert_path)
        self.api_client = ApiClient(configuration=client_configuration)

        self._client_id = client_id
        self._client_secret = client_secret
        self._oauth_scope = oauth_scope
        self._token_url = token_url
        self._cert_path = cert_path

        self._api_client = ApiClient()
        self._order_api = OrderApi(self.api_client)
        self._dataset_api = DatasetApi(self.api_client)

        api_token = self.__fetch_token(
            self._client_id, self._client_secret, self._oauth_scope, self._token_url, self._cert_path
        )
        self.api_client.configuration.api_key["bearer"] = f"Bearer {api_token}"

        refresh_thread = threading.Thread(
            target=lambda: self.__refresh_token_periodically(self._TOKEN_EXPIRATION_MINUTES - 1)
        )
        refresh_thread.daemon = True
        refresh_thread.start()

    def __refresh_token_periodically(self, refresh_interval):
        while True:
            new_token = self.__fetch_token(
                self._client_id, self._client_secret, self._oauth_scope, self._token_url, self._cert_path
            )
            self.api_client.configuration.api_key["bearer"] = f"Bearer {new_token}"

            time.sleep(refresh_interval)

    @property
    def order_api(self) -> OrderApi:
        return self._order_api

    @property
    def dataset_api(self) -> DatasetApi:
        return self._dataset_api

    @property
    def validation_api(self) -> ValidationApi:
        return ValidationApi(self.api_client)

    @property
    def rapid_api(self) -> RapidApi:
        return RapidApi(self.api_client)

    @property
    def campaign_api(self) -> CampaignApi:
        return CampaignApi(self.api_client)

    @property
    def pipeline_api(self) -> PipelineApi:
        return PipelineApi(self.api_client)

    @property
    def workflow_api(self) -> WorkflowApi:
        return WorkflowApi(self.api_client)

    @staticmethod
    def __fetch_token(client_id: str, client_secret: str, scope: str, token_url: str, cert_path: str | None = None) -> str:
        try:
            return requests.post(
                token_url,
                data={
                    'grant_type': 'client_credentials',
                    'client_id': client_id,
                    'client_secret': client_secret,
                    'scope': scope,
                },
                verify=cert_path
            ).json()['access_token']
        except requests.RequestException as e:
            raise Exception(f"Failed to fetch token: {e}")
        except json.JSONDecodeError as e:
            raise Exception(f"Failed to parse token response: {e}")
        except KeyError as e:
            raise Exception(f"Failed to extract token from response: {e}")




from rapidata.api_client.models.capped_selection import (
    CappedSelection as CappedSelectionModel,
)
from rapidata.api_client.models.capped_selection_selections_inner import (
    CappedSelectionSelectionsInner,
)
from rapidata.rapidata_client.selection.base_selection import Selection
from typing import Sequence


class CappedSelection(Selection):

    def __init__(self, selections: Sequence[Selection], max_rapids: int):
        self.selections = selections
        self.max_rapids = max_rapids

    def to_model(self):
        return CappedSelectionModel(
            _t="CappedSelection",
            selections=[
                CappedSelectionSelectionsInner(selection.to_model())
                for selection in self.selections
            ],
            max_rapids=self.max_rapids,
        )



from typing import Any
from rapidata.rapidata_client.referee.base_referee import Referee
from rapidata.api_client.models.early_stopping_referee_model import (
    EarlyStoppingRefereeModel,
)


class EarlyStoppingReferee(Referee):
    """A referee that stops the task when confidence in the winning category exceeds a threshold.

    This referee implements an early stopping mechanism for classification tasks.
    It terminates the task when the confidence in the leading category surpasses
    a specified threshold or when the maximum number of votes is reached.

    The threshold behaves logarithmically, meaning small increments (e.g., from 0.99
    to 0.999) can significantly impact the stopping criteria.

    This referee is supported for the classification and compare tasks.

    Attributes:
        threshold (float): The confidence threshold for early stopping.
        max_vote_count (int): The maximum number of votes allowed before stopping.
    """

    def __init__(self, threshold: float = 0.999, max_vote_count: int = 100):
        """Initialize the ClassifyEarlyStoppingReferee.

        Args:
            threshold (float, optional): The confidence threshold for early stopping.
                Defaults to 0.999.
            max_vote_count (int, optional): The maximum number of votes allowed
                before stopping. Defaults to 100.
        """
        self.threshold = threshold
        self.max_vote_count = max_vote_count

    def to_dict(self):
        return {
            "_t": "ProbabilisticAttachCategoryRefereeConfig",
            "threshold": self.threshold,
            "maxVotes": self.max_vote_count,
        }

    def to_model(self) -> Any:
        return EarlyStoppingRefereeModel(
            _t="EarlyStoppingReferee",
            threshold=self.threshold,
            maxVotes=self.max_vote_count,
        )



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.capped_selection_selections_inner import CappedSelectionSelectionsInner
from rapidata.api_client.models.create_order_model_user_filters_inner import CreateOrderModelUserFiltersInner
from rapidata.api_client.models.feature_flag import FeatureFlag
from typing import Optional, Set
from typing_extensions import Self

class AddCampaignModel(BaseModel):
    """
    The model for adding a campaign to an existing pipeline.
    """ # noqa: E501
    t: StrictStr = Field(description="Discriminator value for AddCampaignModel", alias="_t")
    pipeline_id: StrictStr = Field(description="The pipeline the campaign should belong to.", alias="pipelineId")
    artifact_id: Optional[StrictStr] = Field(default=None, description="The id of the campaign artifact. If not provided a random value will be used.", alias="artifactId")
    campaign_name: StrictStr = Field(description="The name of the campaign.", alias="campaignName")
    user_filters: List[CreateOrderModelUserFiltersInner] = Field(description="The user filters to apply to the campaign.", alias="userFilters")
    validation_set_id: Optional[StrictStr] = Field(default=None, description="A validation set that should be used.", alias="validationSetId")
    selections: Optional[List[CappedSelectionSelectionsInner]] = Field(description="The selections that the campaign should have.")
    feature_flags: List[FeatureFlag] = Field(description="The feature flags that should be applied to the campaign.", alias="featureFlags")
    priority: StrictInt = Field(description="The priority of the campaign.")
    __properties: ClassVar[List[str]] = ["_t", "pipelineId", "artifactId", "campaignName", "userFilters", "validationSetId", "selections", "featureFlags", "priority"]

    @field_validator('t')
    def t_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['AddCampaignModel']):
            raise ValueError("must be one of enum values ('AddCampaignModel')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AddCampaignModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in user_filters (list)
        _items = []
        if self.user_filters:
            for _item_user_filters in self.user_filters:
                if _item_user_filters:
                    _items.append(_item_user_filters.to_dict())
            _dict['userFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in selections (list)
        _items = []
        if self.selections:
            for _item_selections in self.selections:
                if _item_selections:
                    _items.append(_item_selections.to_dict())
            _dict['selections'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if artifact_id (nullable) is None
        # and model_fields_set contains the field
        if self.artifact_id is None and "artifact_id" in self.model_fields_set:
            _dict['artifactId'] = None

        # set to None if validation_set_id (nullable) is None
        # and model_fields_set contains the field
        if self.validation_set_id is None and "validation_set_id" in self.model_fields_set:
            _dict['validationSetId'] = None

        # set to None if selections (nullable) is None
        # and model_fields_set contains the field
        if self.selections is None and "selections" in self.model_fields_set:
            _dict['selections'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AddCampaignModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_t": obj.get("_t") if obj.get("_t") is not None else 'AddCampaignModel',
            "pipelineId": obj.get("pipelineId"),
            "artifactId": obj.get("artifactId"),
            "campaignName": obj.get("campaignName"),
            "userFilters": [CreateOrderModelUserFiltersInner.from_dict(_item) for _item in obj["userFilters"]] if obj.get("userFilters") is not None else None,
            "validationSetId": obj.get("validationSetId"),
            "selections": [CappedSelectionSelectionsInner.from_dict(_item) for _item in obj["selections"]] if obj.get("selections") is not None else None,
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None,
            "priority": obj.get("priority")
        })
        return _obj





from time import sleep
from rapidata.rapidata_client.dataset.rapidata_dataset import RapidataDataset
from rapidata.service.openapi_service import OpenAPIService
import json
from rapidata.api_client.exceptions import ApiException
from typing import Optional, cast, Any
from rapidata.api_client.models.workflow_artifact_model import WorkflowArtifactModel
from tqdm import tqdm

class RapidataOrder:
    """
    Represents a Rapidata order.

    Args:
        The ID of the order.
        The optional Dataset associated with the order.
        The OpenAPIService instance used to interact with the Rapidata API.
        The name of the order.
    """

    def __init__(
        self,
        order_id: str,
        dataset: Optional[RapidataDataset],
        openapi_service: OpenAPIService,
        name: str,
    ):
        self.openapi_service = openapi_service
        self.order_id = order_id
        self._dataset = dataset
        self.name = name
        self._workflow_id = None

    def submit(self):
        """
        Submits the order for processing.
        """
        self.openapi_service.order_api.order_submit_post(self.order_id)

    def get_status(self) -> str:
        """
        Gets the status of the order.
        
        Returns: 
            The status of the order.
        """
        return self.openapi_service.order_api.order_get_by_id_get(self.order_id).state

    def display_progress_bar(self, refresh_rate=5):
        """
        Displays a progress bar for the order processing using tqdm.
        
        Prameter: 
            How often to refresh the progress bar, in seconds.
        """
        total_rapids = self._get_workflow_progress().total
        with tqdm(total=total_rapids, desc="Processing order", unit="rapids") as pbar:
            completed_rapids = 0
            while True:
                current_completed = self._get_workflow_progress().completed
                if current_completed > completed_rapids:
                    pbar.update(current_completed - completed_rapids)
                    completed_rapids = current_completed

                if completed_rapids >= total_rapids:
                    break

                sleep(refresh_rate)

    def _get_workflow_id(self):
        if self._workflow_id:
            return self._workflow_id

        for _ in range(2):
            try:
                order_result = self.openapi_service.order_api.order_get_by_id_get(self.order_id)
                pipeline = self.openapi_service.pipeline_api.pipeline_id_get(order_result.pipeline_id)
                self._workflow_id = cast(WorkflowArtifactModel, pipeline.artifacts["workflow-artifact"].actual_instance).workflow_id
                break
            except Exception:
                sleep(2)
        if not self._workflow_id:
            raise Exception("Order has not started yet. Please start it or wait for a few seconds and try again.")
        return self._workflow_id
    
    def _get_workflow_progress(self):
        workflow_id = self._get_workflow_id()
        progress = None
        for _ in range(2):
            try:
                progress = self.openapi_service.workflow_api.workflow_get_progress_get(workflow_id)
                break
            except Exception:
                sleep(5)

        if not progress:
            raise Exception(f"Failed to get progress. Please try again in a few seconds.")
        
        return progress

        
    def get_results(self) -> dict[str, Any]:
        """
        Gets the results of the order. 
        If the order is still processing, this method will block until the order is completed and then return the results.

        Returns: 
            The results of the order.
        """
        while self.get_status() == "Processing":
            sleep(5)

        try:
            # Get the raw result string
            result_str = self.openapi_service.order_api.order_result_get(id=self.order_id)
            # Parse the result string as JSON
            return json.loads(result_str)
        except ApiException as e:
            # Handle API exceptions
            raise Exception(f"Failed to get order results: {str(e)}") from e
        except json.JSONDecodeError as e:
            # Handle JSON parsing errors
            raise Exception(f"Failed to parse order results: {str(e)}") from e

    @property
    def dataset(self) -> RapidataDataset | None:
        """
        The dataset associated with the order.
        Returns: 
            The RapidataDataset instance.
        """
        return self._dataset



from .base_metadata import Metadata
from .private_text_metadata import PrivateTextMetadata
from .public_text_metadata import PublicTextMetadata
from .prompt_metadata import PromptMetadata
from .transcription_metadata import TranscriptionMetadata



from rapidata.api_client.models.naive_referee_model import NaiveRefereeModel
from rapidata.rapidata_client.referee.base_referee import Referee


class NaiveReferee(Referee):
    """A simple referee that completes a task after a fixed number of responses.

    This referee implements a straightforward approach to task completion,
    where the task is considered finished after a predetermined number of
    responses have been made, regardless of the content or quality of those responses.

    Attributes:
        responses (int): The number of responses required to complete the task.
    """

    def __init__(self, responses: int = 10):
        """Initialize the NaiveReferee.

        Args:
            responses (int, optional): The number of responses required
                to complete the task. Defaults to 10. This is per media item.
        """
        super().__init__()
        self.responses = responses

    def to_dict(self):
        return {
            "_t": "NaiveRefereeConfig",
            "guessesRequired": self.responses,
        }

    def to_model(self):
        return NaiveRefereeModel(_t="NaiveReferee", totalVotes=self.responses)



from rapidata.rapidata_client.dataset.rapidata_validation_set import (
    RapidataValidationSet,
)
from rapidata.rapidata_client.dataset.validation_set_builder import ValidationSetBuilder
from rapidata.rapidata_client.order.rapidata_order_builder import RapidataOrderBuilder
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.order.rapidata_order import RapidataOrder
from rapidata.rapidata_client.dataset.rapidata_dataset import RapidataDataset

from rapidata.rapidata_client.simple_builders.simple_classification_builders import ClassificationQuestionBuilder
from rapidata.rapidata_client.simple_builders.simple_compare_builders import CompareCriteriaBuilder

from rapidata.api_client.exceptions import BadRequestException
from urllib3._collections import HTTPHeaderDict

from rapidata.api_client.models.query_orders_model import QueryOrdersModel
from rapidata.api_client.models.page_info import PageInfo
from rapidata.api_client.models.root_filter import RootFilter
from rapidata.api_client.models.filter import Filter
from rapidata.api_client.models.sort_criterion import SortCriterion

from rapidata.api_client.models.query_validation_set_model import QueryValidationSetModel


class RapidataClient:
    """The Rapidata client is the main entry point for interacting with the Rapidata API. It allows you to create orders and validation sets. For creating a new order, check out `new_order()`. For creating a new validation set, check out `new_validation_set()`."""

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        endpoint: str = "https://api.rapidata.ai",
        token_url: str = "https://auth.rapidata.ai/connect/token",
        oauth_scope: str = "openid",
        cert_path: str | None = None,
    ):
        """Initialize the RapidataClient. Best practice is to store the client ID and client secret in environment variables. Ask your Rapidata representative for the client ID and client secret.

        Args:
            client_id (str): The client ID for authentication.
            client_secret (str): The client secret for authentication.
            endpoint (str, optional): The API endpoint URL.
        """
        self.openapi_service = OpenAPIService(
            client_id=client_id,
            client_secret=client_secret,
            endpoint=endpoint,
            token_url=token_url,
            oauth_scope=oauth_scope,
            cert_path=cert_path
        )

    def new_order(self, name: str) -> RapidataOrderBuilder:
        """Create a new order using a RapidataOrderBuilder instance.

        Args:
            name (str): The name of the order.

        Returns:
            RapidataOrderBuilder: A RapidataOrderBuilder instance.
        """
        return RapidataOrderBuilder(openapi_service=self.openapi_service, name=name)

    def new_validation_set(self, name: str) -> ValidationSetBuilder:
        """Create a new validation set using a ValidationDatasetBuilder instance.

        Args:
            name (str): The name of the validation set.

        Returns:
            ValidationSetBuilder: A ValidationDatasetBuilder instance.
        """
        return ValidationSetBuilder(name=name, openapi_service=self.openapi_service)

    def get_order(self, order_id: str) -> RapidataOrder:
        """Get an order by ID.

        Args:
            order_id (str): The ID of the order.

        Returns:
            RapidataOrder: The Order instance.
        """

        # TODO: check the pipeline for the dataset id - not really necessary atm
        # order = self.openapi_service.order_api.order_get_by_id_get(order_id)
        # pipeline = self.openapi_service..pipeline_get_by_id_get(order.pipeline_id)
        try:
            order = self.openapi_service.order_api.order_get_by_id_get(order_id)
        except Exception:
            raise ValueError(f"Order with ID {order_id} not found.")

        temp_dataset = RapidataDataset("temp", self.openapi_service)
        return RapidataOrder(
            dataset=temp_dataset, 
            order_id=order_id, 
            name=order.order_name,
            openapi_service=self.openapi_service)

    def find_orders(self, name: str = "", amount: int = 1) -> list[RapidataOrder]:
        """Find your recent orders given criteria. If nothing is provided, it will return the most recent order.

        Args:
            name (str, optional): The name of the order - matching order will contain the name. Defaults to "" for any order.
            amount (int, optional): The amount of orders to return. Defaults to 1.

        Returns:
            list[RapidataOrder]: A list of RapidataOrder instances.
        """
        try:
            order_page_result = self.openapi_service.order_api.order_query_get(QueryOrdersModel(
                page=PageInfo(index=1, size=amount),
                filter=RootFilter(filters=[Filter(field="OrderName", operator="Contains", value=name)]),
                sortCriteria=[SortCriterion(direction="Desc", propertyName="OrderDate")]
                ))

        except BadRequestException as e:
            raise ValueError(f"Error occured during request. \nError: {e.body} \nTraceid: {e.headers.get('X-Trace-Id') if isinstance(e.headers, HTTPHeaderDict) else 'Unknown'}")

        except Exception as e:
            raise ValueError(f"Unknown error occured: {e}")

        orders = [self.get_order(order.id) for order in order_page_result.items]
        return orders
    
    def get_validation_set(self, validation_set_id: str) -> RapidataValidationSet:
        """Get a validation set by ID.

        Args:
            validation_set_id (str): The ID of the validation set.

        Returns:
            RapidataValidationSet: The ValidationSet instance.
        """
        try:
            validation_set = self.openapi_service.validation_api.validation_get_by_id_get(id=validation_set_id)
        except Exception:
            raise ValueError(f"ValidationSet with ID {validation_set_id} not found.")
        
        return RapidataValidationSet(validation_set_id, self.openapi_service, validation_set.name)
    
    def find_validation_sets(self, name: str = "", amount: int = 1) -> list[RapidataValidationSet]:
        try:
            validation_page_result = self.openapi_service.validation_api.validation_query_validation_sets_get(QueryValidationSetModel(
                pageInfo=PageInfo(index=1, size=amount),
                filter=RootFilter(filters=[Filter(field="Name", operator="Contains", value=name)]),
                sortCriteria=[SortCriterion(direction="Desc", propertyName="CreatedAt")]
                ))

        except BadRequestException as e:
            raise ValueError(f"Error occured during request. \nError: {e.body} \nTraceid: {e.headers.get('X-Trace-Id') if isinstance(e.headers, HTTPHeaderDict) else 'Unknown'}")

        except Exception as e:
            raise ValueError(f"Unknown error occured: {e}")

        orders = [self.get_validation_set(validation_set.id) for validation_set in validation_page_result.items] # type: ignore # will be fixed with the next backend deployment
        return orders

    def create_classify_order(self, name: str) -> ClassificationQuestionBuilder:
        """Create a new classification order where people are asked to classify an image.

        Args:
            name (str): The name of the order.

        Returns:
            ClassificationQuestionBuilder: A ClassificationQuestionBuilder instance.
        """
        return ClassificationQuestionBuilder(name=name, openapi_service=self.openapi_service)

    def create_compare_order(self, name: str) -> CompareCriteriaBuilder:
        """Create a new comparison order where people are asked to compare two images.

        Args:
            name (str): The name of the order.

        Returns:
            CompareQuestionBuilder: A CompareQuestionBuilder instance.
        """
        return CompareCriteriaBuilder(name=name, openapi_service=self.openapi_service)



from .rapidata_client import RapidataClient
from .workflow import (
    ClassifyWorkflow,
    TranscriptionWorkflow,
    CompareWorkflow,
    FreeTextWorkflow,
)
from .selection import (
    DemographicSelection,
    LabelingSelection,
    ValidationSelection,
    ConditionalValidationSelection,
    CappedSelection,
)
from .referee import NaiveReferee, EarlyStoppingReferee
from .metadata import (
    PrivateTextMetadata,
    PublicTextMetadata,
    PromptMetadata,
    TranscriptionMetadata,
)
from .settings import Settings, FeatureFlags # remove FeatureFlags next major version
from .country_codes import CountryCodes
from .assets import MediaAsset, TextAsset, MultiAsset
from .filter import (
    CountryFilter,
    LanguageFilter,
    UserScoreFilter,
    CampaignFilter,
    AgeFilter,
    AgeGroup,
    GenderFilter,
    Gender,
)



from typing import Any
from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel
from rapidata.api_client.models.simple_workflow_model_blueprint import SimpleWorkflowModelBlueprint
from rapidata.rapidata_client.workflow import Workflow
from rapidata.api_client.models.free_text_rapid_blueprint import FreeTextRapidBlueprint


class FreeTextWorkflow(Workflow):
    """
    A workflow for free text input tasks.

    This class represents a workflow where users can provide free-form text responses
    to a given question.

    Attributes:
        _question (str): The question to be answered with free text.

    Args:
        question (str): The question to be presented for free text input.
    """

    def __init__(self, question: str):
        """
        Initialize a FreeTextWorkflow instance.

        Args:
            question (str): The question to be presented for free text input.
        """
        super().__init__(type="SimpleWorkflowConfig")
        self._question = question

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "blueprint": {
                "_t": "FreeTextBlueprint",
                "question": self._question,
            },
        }

    def to_model(self) -> SimpleWorkflowModel:
        blueprint = FreeTextRapidBlueprint(
            _t="FreeTextBlueprint",
            question=self._question,
        )

        return SimpleWorkflowModel(
            _t="SimpleWorkflow",
            blueprint=SimpleWorkflowModelBlueprint(blueprint),
        )



from typing import Any
from rapidata.api_client.models.simple_workflow_model_blueprint import SimpleWorkflowModelBlueprint
from rapidata.rapidata_client.workflow import Workflow
from rapidata.api_client.models.compare_rapid_blueprint import CompareRapidBlueprint
from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel


class CompareWorkflow(Workflow):
    """
    A workflow for comparison tasks.

    This class represents a comparison workflow where items are compared based on
    specified criteria.

    Attributes:
        _criteria (str): The criteria used for comparison.

    Args:
        criteria (str): The criteria to be used for comparison.
    """

    def __init__(self, criteria: str):
        """
        Initialize a CompareWorkflow instance.

        Args:
            criteria (str): The criteria to be used for comparison.
        """
        super().__init__(type="CompareWorkflowConfig")
        self._criteria = criteria

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "criteria": self._criteria,
        }

    def to_model(self) -> SimpleWorkflowModel:
        blueprint = CompareRapidBlueprint(
            _t="CompareBlueprint",
            criteria=self._criteria,
        )

        return SimpleWorkflowModel(
            _t="SimpleWorkflow",
            blueprint=SimpleWorkflowModelBlueprint(blueprint),
        )



from rapidata.api_client.models.evaluation_workflow_model import EvaluationWorkflowModel
from rapidata.rapidata_client.workflow.base_workflow import Workflow


class EvaluationWorkflow(Workflow):
    """Workflow to run evaluation orders. This is used internally only and should not be necessary to be used by clients."""

    def __init__(self, validation_set_id: str):
        super().__init__("EvaluationWorkflow")
        self.validation_set_id = validation_set_id

    def to_model(self):
        return EvaluationWorkflowModel(
            _t="EvaluationWorkflow", validationSetId=self.validation_set_id
        )



from .base_workflow import Workflow 
from .classify_workflow import ClassifyWorkflow
from .compare_workflow import CompareWorkflow
from .free_text_workflow import FreeTextWorkflow
from .transcription_workflow import TranscriptionWorkflow
from .evaluation_workflow import EvaluationWorkflow



from typing import Any
from rapidata.rapidata_client.filter.base_filter import Filter
from rapidata.api_client.models.user_score_user_filter_model import (
    UserScoreUserFilterModel,
)


class UserScoreFilter(Filter):

    def __init__(self, lower_bound: int = 0, upper_bound: int = 1):
        self.upper_bound = upper_bound
        self.lower_bound = lower_bound

    def to_model(self):
        return UserScoreUserFilterModel(
            _t="UserScoreFilter",
            upperbound=self.upper_bound,
            lowerbound=self.lower_bound,
        )



import os
from rapidata.api_client.models.attach_category_truth import AttachCategoryTruth
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.compare_payload import ComparePayload
from rapidata.api_client.models.compare_truth import CompareTruth
from rapidata.api_client.models.transcription_payload import TranscriptionPayload
from rapidata.api_client.models.transcription_truth import TranscriptionTruth
from rapidata.api_client.models.transcription_word import TranscriptionWord
from rapidata.rapidata_client.assets.media_asset import MediaAsset
from rapidata.rapidata_client.assets.multi_asset import MultiAsset
from rapidata.rapidata_client.assets.text_asset import TextAsset
from rapidata.rapidata_client.dataset.rapidata_validation_set import (
    RapidataValidationSet,
)
from rapidata.rapidata_client.dataset.validation_rapid_parts import ValidatioRapidParts
from rapidata.rapidata_client.metadata.base_metadata import Metadata
from rapidata.service.openapi_service import OpenAPIService


class ValidationSetBuilder:
    """The ValidationSetBuilder is used to build a validation set.
    Give the validation set a name and then add classify, compare, or transcription rapid parts to it.
    Get a `ValidationSetBuilder` by calling [`rapi.new_validation_set()`](../rapidata_client.md/#rapidata.rapidata_client.rapidata_client.RapidataClient.new_validation_set).
    """

    def __init__(self, name: str, openapi_service: OpenAPIService):
        """
        Initialize the ValidationSetBuilder.

        Args:
            name (str): The name of the validation set.
            openapi_service (OpenAPIService): An instance of OpenAPIService to interact with the API.
        """
        self.name = name
        self.openapi_service = openapi_service
        self.validation_set_id: str | None = None
        self._rapid_parts: list[ValidatioRapidParts] = []

    def create(self) -> RapidataValidationSet:
        """Create the validation set by executing all HTTP requests. This should be the last method called on the builder.

        Returns:
            RapidataValidationSet: A RapidataValidationSet instance.

        Raises:
            ValueError: If the validation set creation fails.
        """
        result = (
            self.openapi_service.validation_api.validation_create_validation_set_post(
                name=self.name
            )
        )
        self.validation_set_id = result.validation_set_id

        if self.validation_set_id is None:
            raise ValueError("Failed to create validation set")

        validation_set = RapidataValidationSet(
            validation_set_id=self.validation_set_id,
            openapi_service=self.openapi_service,
            name=self.name,
        )

        for rapid_part in self._rapid_parts:
            validation_set.add_general_validation_rapid(
                payload=rapid_part.payload,
                truths=rapid_part.truths,
                metadata=rapid_part.metadata,
                asset=rapid_part.asset,
                randomCorrectProbability=rapid_part.randomCorrectProbability,
            )

        return validation_set

    def add_classify_rapid(
        self,
        asset: MediaAsset | TextAsset,
        question: str,
        categories: list[str],
        truths: list[str],
        metadata: list[Metadata] = [],
    ):
        """Add a classify rapid to the validation set.

        Args:
            asset (MediaAsset | TextAsset): The asset for the rapid.
            question (str): The question for the rapid.
            categories (list[str]): The list of categories for the rapid.
            truths (list[str]): The list of truths for the rapid.
            metadata (list[Metadata], optional): The metadata for the rapid. Defaults to an empty list.

        Returns:
            ValidationSetBuilder: The ValidationSetBuilder instance.

        Raises:
            ValueError: If the lengths of categories and truths are inconsistent.
        """
        payload = ClassifyPayload(
            _t="ClassifyPayload", possibleCategories=categories, title=question
        )
        model_truth = AttachCategoryTruth(
            correctCategories=truths, _t="AttachCategoryTruth"
        )

        self._rapid_parts.append(
            ValidatioRapidParts(
                question=question,
                payload=payload,
                truths=model_truth,
                metadata=metadata,
                randomCorrectProbability=len(truths) / len(categories),
                asset=asset,
            )
        )

        return self

    def add_compare_rapid(
        self,
        asset: MultiAsset,
        question: str,
        truth: str,
        metadata: list[Metadata] = [],
    ):
        """Add a compare rapid to the validation set.

        Args:
            asset (MultiAsset): The assets for the rapid.
            question (str): The question for the rapid.
            truth (str): The truth identifier for the rapid.
            metadata (list[Metadata], optional): The metadata for the rapid. Defaults to an empty list.

        Returns:
            ValidationSetBuilder: The ValidationSetBuilder instance.

        Raises:
            ValueError: If the number of assets is not exactly two.
        """
        payload = ComparePayload(_t="ComparePayload", criteria=question)
        # take only last part of truth path
        truth = os.path.basename(truth)
        model_truth = CompareTruth(_t="CompareTruth", winnerId=truth)

        if len(asset) != 2:
            raise ValueError("Compare rapid requires exactly two media paths")

        self._rapid_parts.append(
            ValidatioRapidParts(
                question=question,
                payload=payload,
                truths=model_truth,
                metadata=metadata,
                randomCorrectProbability=1 / len(asset),
                asset=asset,
            )
        )

        return self

    def add_transcription_rapid(
        self,
        asset: MediaAsset | TextAsset,
        question: str,
        transcription: str,
        truths: list[int],
        strict_grading: bool | None = None,
        metadata: list[Metadata] = [],
    ):
        """Add a transcription rapid to the validation set.

        Args:
            asset (MediaAsset | TextAsset): The asset for the rapid.
            question (str): The question for the rapid.
            transcription (list[str]): The transcription for the rapid.
            truths (list[int]): The list of indices of the true word selections.
            strict_grading (bool | None, optional): The strict grading for the rapid. Defaults to None.
            metadata (list[Metadata], optional): The metadata for the rapid.

        Returns:
            ValidationSetBuilder: The ValidationSetBuilder instance.

        Raises:
            ValueError: If a correct word is not found in the transcription.
        """
        transcription_words = [
            TranscriptionWord(word=word, wordIndex=i)
            for i, word in enumerate(transcription.split())
        ]

        true_words = []
        for idx in truths:
            if idx > len(transcription_words) - 1:
                raise ValueError(f"Index {idx} is out of bounds")
            true_words.append(transcription_words[idx])

        payload = TranscriptionPayload(
            _t="TranscriptionPayload", title=question, transcription=transcription_words
        )

        model_truth = TranscriptionTruth(
            _t="TranscriptionTruth",
            correctWords=true_words,
            strictGrading=strict_grading,
        )

        self._rapid_parts.append(
            ValidatioRapidParts(
                question=question,
                asset=asset,
                payload=payload,
                truths=model_truth,
                metadata=metadata,
                randomCorrectProbability = 1 / len(transcription_words),
            )
        )

        return self



from rapidata.rapidata_client.order.rapidata_order_builder import RapidataOrderBuilder
from rapidata.rapidata_client.metadata.base_metadata import Metadata
from rapidata.rapidata_client.referee.naive_referee import NaiveReferee
from rapidata.rapidata_client.referee.early_stopping_referee import EarlyStoppingReferee
from rapidata.rapidata_client.selection.base_selection import Selection
from rapidata.rapidata_client.workflow.classify_workflow import ClassifyWorkflow
from rapidata.rapidata_client.selection.validation_selection import ValidationSelection
from rapidata.rapidata_client.selection.labeling_selection import LabelingSelection
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.assets import MediaAsset
from typing import Sequence

class ClassificationOrderBuilder:
    def __init__(self, name: str, question: str, options: list[str], media_paths: list[str], openapi_service: OpenAPIService):
        self._order_builder = RapidataOrderBuilder(name=name, openapi_service=openapi_service)
        self._question = question
        self._options = options
        self._media_paths = media_paths
        self._responses_required = 10
        self._probability_threshold = None
        self._metadata = None
        self._validation_set_id = None

    def metadata(self, metadata: Sequence[Metadata]):
        """Set the metadata for the classification order. Has to be the same lenght as the media paths."""
        self._metadata = metadata
        return self

    def responses(self, responses_required: int):
        """Set the number of responses required for the classification order."""
        self._responses_required = responses_required
        return self

    def probability_threshold(self, probability_threshold: float):
        """Set the probability threshold for early stopping."""
        self._probability_threshold = probability_threshold
        return self

    def validation_set_id(self, validation_set_id: str):
        """Set the validation set ID for the classification order."""
        self._validation_set_id = validation_set_id
        return self

    def create(self, submit: bool = True, max_upload_workers: int = 10):
        if self._probability_threshold and self._responses_required:
            referee = EarlyStoppingReferee(
                max_vote_count=self._responses_required,
                threshold=self._probability_threshold
            )

        else:
            referee = NaiveReferee(responses=self._responses_required)

        assets = [MediaAsset(path=media_path) for media_path in self._media_paths]

        selection: list[Selection] = ([ValidationSelection(amount=1, validation_set_id=self._validation_set_id), LabelingSelection(amount=2)] 
                     if self._validation_set_id 
                     else [LabelingSelection(amount=3)])

        order = (self._order_builder
            .workflow(
                ClassifyWorkflow(
                    question=self._question,
                    options=self._options
                )
            )
            .referee(referee)
            .media(assets, metadata=self._metadata) # type: ignore
            .selections(selection)
            .create(submit=submit, max_workers=max_upload_workers))

        return order 


class ClassificationMediaBuilder:
    "test"
    def __init__(self, name: str, question: str, options: list[str], openapi_service: OpenAPIService):
        self._openapi_service = openapi_service
        self._name = name
        self._question = question
        self._options = options
        self._media_paths = None

    def media(self, media_paths: list[str]) -> ClassificationOrderBuilder:
        """Set the media assets for the classification order by providing the local paths to the files."""
        self._media_paths = media_paths
        return self._build()

    def _build(self) -> ClassificationOrderBuilder:
        if self._media_paths is None:
            raise ValueError("Media paths are required")
        return ClassificationOrderBuilder(self._name, self._question, self._options, self._media_paths, openapi_service=self._openapi_service)


class ClassificationOptionsBuilder:
    def __init__(self, name: str, question: str, openapi_service: OpenAPIService):
        self._openapi_service = openapi_service
        self._name = name
        self._question = question
        self._options = None

    def options(self, options: list[str]) -> ClassificationMediaBuilder:
        """Set the answer options for the classification order."""
        self._options = options
        return self._build()

    def _build(self) -> ClassificationMediaBuilder:
        if self._options is None:
            raise ValueError("Options are required")
        return ClassificationMediaBuilder(self._name, self._question, self._options, self._openapi_service)


class ClassificationQuestionBuilder:
    def __init__(self, name: str, openapi_service: OpenAPIService):
        self._openapi_service = openapi_service
        self._name = name
        self._question = None

    def question(self, question: str) -> ClassificationOptionsBuilder:
        """Set the question for the classification order."""
        self._question = question
        return self._build()

    def _build(self) -> ClassificationOptionsBuilder:
        if self._question is None:
            raise ValueError("Question is required")
        return ClassificationOptionsBuilder(self._name, self._question, self._openapi_service)




from rapidata.api_client.models.validation_selection import ValidationSelection as ValidationSelectionModel
from rapidata.rapidata_client.selection.base_selection import Selection


class ValidationSelection(Selection):

    def __init__(self, validation_set_id: str, amount: int = 1):
        self.validation_set_id = validation_set_id
        self.amount = amount


    def to_model(self):
        return ValidationSelectionModel(_t="ValidationSelection", validationSetId=self.validation_set_id, amount=self.amount)
    


from typing import Any
from rapidata.api_client.models.attach_category_rapid_blueprint import AttachCategoryRapidBlueprint
from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel
from rapidata.api_client.models.simple_workflow_model_blueprint import SimpleWorkflowModelBlueprint
from rapidata.rapidata_client.workflow import Workflow


class ClassifyWorkflow(Workflow):
    """
    A workflow for classification tasks.

    This class represents a classification workflow where a question is presented
    along with a list of possible options for classification.

    Args:
        question (str): The classification question to be presented.
        options (list[str]): A list of possible classification options.

    Attributes:
        _question (str): The classification question.
        _options (list[str]): The list of classification options.
    """

    def __init__(self, question: str, options: list[str]):
        super().__init__(type="SimpleWorkflowConfig")
        self._question = question
        self._options = options

    def to_dict(self) -> dict[str, Any]:
        return {
            **super().to_dict(),
            "blueprint": {
                "_t": "ClassifyBlueprint",
                "title": self._question,
                "possibleCategories": self._options,
            }
        }

    def to_model(self) -> SimpleWorkflowModel:
        blueprint = AttachCategoryRapidBlueprint(
            _t="ClassifyBlueprint",
            title=self._question,
            possibleCategories=self._options,
        )

        return SimpleWorkflowModel(
            _t="SimpleWorkflow",
            blueprint=SimpleWorkflowModelBlueprint(blueprint),
        )



from rapidata.rapidata_client.selection.base_selection import Selection
from rapidata.api_client.models.demographic_selection import DemographicSelection as DemographicSelectionModel


class DemographicSelection(Selection):
    """Demographic selection class."""

    def __init__(self, keys: list[str], maxRapids: int):
        self.keys = keys
        self.maxRapids = maxRapids

    def to_model(self):
        return DemographicSelectionModel(_t="DemographicSelection", keys=self.keys, maxRapids=self.maxRapids)
